/*
 * alarm_pir2.c
 *
 * Created: 2016-06-21 08:50:41
 *  Author: dsosnowski
 */ 



#define F_CPU 8000000UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "hard.h"
#include "wdog.h"
#include "ee_mem.h"
#include "scanner.h"


//--------------------------------------------------
void HardInit(void);
void AnalogInit(void);
char AnalogGet(char channel);

char Scan(void);
char InputGet(char channel);
void LedProc(void);
void OutProc(void);
void BuzzProc(char time);
char ConfigGet(char mask);
char UserGet(char act);


#define MODE_IDLE				1
#define MODE_TO_ARM				2
#define MODE_ARMED				3				
#define MODE_LEARN				4			

#define MAIN_DELAY				20	
#define LED_SHORT				6
#define LED_LONG				25
#define LED_PERIOD				255
#define IR_HOLD					50
#define ALARM_MIN_TIME			3
#define SHORT_BUZZ				3
#define DISARM_BUZZ				30

#define PROG_BIT				1
#define MODULO_BIT				2
#define START_BIT				4

#define NUM_OF_INPUTS			7
#define DETECT_CHANNEL			1
#define SW_CHANNEL				2
#define REM_CHANNEL				3
#define CFG_CHANNEL				4
#define ALARM_TIM_CHANNEL		5
#define ARMED_TIM_CHANNEL		6


#define INPUT_THRESHOLD			6

struct {
	char mode;
	char state;
	char led;
	char ir_hold;
	uint16_t armedtime;
	char alarmgo;
	char alarm;
	uint16_t alarmtime;
	char buzz;
	char disarm;
} system;

char inputs[NUM_OF_INPUTS + 1];



//--------------------------------------------------

int main(void)
{
	char action;
	char next;
	uint16_t temp;
	char startup = 100;
	
	HardInit();
	AnalogInit();
	ScannerInit();
	sei();
	WdtSet(7);

		
	next = MODE_IDLE;
	system.ir_hold = IR_HOLD;
	system.alarmtime = 0xFFFF;
	system.armedtime = 0;
	system.alarmgo = 0;
	system.alarm = 0;
	system.state = 0;
	
    while(1)
    {
		if (startup){
			startup--;
			if (startup == 0) {
				if (ConfigGet(START_BIT) == 1) {
					next = MODE_TO_ARM;
				}
			}
		}
		
		//-----------------------------------------------------------
		if (next) system.mode = next;
		next = 0;
		action = 0;
		
		//-----------------------------------------------------------
		if (system.mode == MODE_IDLE){						
			action = Scan();			
			if (UserGet(action) > 0) {
				next = MODE_TO_ARM;
				system.armedtime = 0;
			}
			if (ConfigGet(PROG_BIT) == 1) next = MODE_LEARN;
						
			if (system.disarm) {
				system.disarm--;
				if ((system.disarm % 10) == 0) BuzzProc(3);
			}
			
			system.state = 0;
			system.alarmtime = 0;
			system.alarmgo = 0;
			system.alarm = 0;
		}
		
		//-----------------------------------------------------------
		if (system.mode == MODE_TO_ARM){
			action = Scan();
			if (UserGet(action) > 0) {
				next = MODE_IDLE;
				system.disarm = DISARM_BUZZ;
			}
			
			temp = (inputs[ARMED_TIM_CHANNEL] << 4);
			if (system.armedtime < temp) {
				system.armedtime++;
				
				if ((system.armedtime % 48) == 0) BuzzProc(SHORT_BUZZ);
				if ((system.armedtime + 255) > temp) {
					if ((system.armedtime % 24) == 0) BuzzProc(SHORT_BUZZ);
				}
				
			} else {
				if (system.alarmgo == 0){
					if (InputGet(DETECT_CHANNEL) == 0) {
						next = MODE_ARMED;
						system.armedtime = 0;
					} else {
						system.armedtime -= 23;
					}
				} else {
					next = MODE_ARMED;
					system.armedtime = 0;
				}
			}
		}

		//-----------------------------------------------------------
		if (system.mode == MODE_ARMED){
			action = Scan();
			if (UserGet(action) > 0) {
				next = MODE_IDLE;
				system.disarm = DISARM_BUZZ;
			}
			
			if (system.alarmgo > 0) {
				system.alarmgo = 0;
				system.alarmtime = 0;
				system.alarm = 1;
			}
			
			if (action == DETECT_CHANNEL) {
				if (system.alarm == 0) {
					system.alarmgo = 1;
					next = MODE_TO_ARM;
				}
			}			
			
			if (system.alarm > 0) {
			
				if (system.alarmtime < (inputs[ALARM_TIM_CHANNEL] << 6)) {
					system.alarmtime++;
					if (ConfigGet(MODULO_BIT) == 1) {
						if (system.alarmtime & 32)
							system.state = 1;
						else
							system.state = 0;
					} else {
						system.state = 1;
					}
		
					if (system.alarmtime & 32) BuzzProc(3);		
				} else {
					system.state = 0;
					system.alarm = 0;
					system.alarmtime = 0;
				}
			}
		}
		
		//-----------------------------------------------------------
		if (system.mode == MODE_LEARN){
			action = Scan();					
			action = ScannerCheck(0);
			if (action == SCAN_COMPLETE) {
				ScannerSave(1);
				system.ir_hold = IR_HOLD;
			}	
			
			if (ConfigGet(PROG_BIT) == 0) next = MODE_IDLE;		
		}
		
		//--------------------------------------------
		if (system.ir_hold > 0) {
			system.led = 0;
			system.ir_hold--;
			if (system.ir_hold == 0) ScannerStart();
		}
		
		//--------------------------------------------
		OutProc();
		LedProc();
		BuzzProc(0);
		_delay_ms(MAIN_DELAY);
		WdtReset();
	}
	
}

//--------------------------------------------------
char Scan(void){
	char state;
	char result = 0;
	char channel;
	
	//---------------------------------------------------------------------
	for(channel = 1; channel <= REM_CHANNEL; channel++){
		if (channel == DETECT_CHANNEL) state = DETECT_GET;
		if (channel == SW_CHANNEL) state = SW_GET;
		if (channel == REM_CHANNEL) state = REM_GET;
	
		if (state == 0){
			if (inputs[channel] < 0xFF) {
				inputs[channel]++;
				if (inputs[channel] == INPUT_THRESHOLD) result = channel;
			}
		} else {
			inputs[channel] = 0;
		}
	}
	
	//-----------------------
	state = CFG_IN;
	state >>= 2;
	inputs[CFG_CHANNEL] = state & 0x07;
	
	//-----------------------
	inputs[ALARM_TIM_CHANNEL] = AnalogGet(CFGA1_PIN);
	if (inputs[ALARM_TIM_CHANNEL] < 3) inputs[ALARM_TIM_CHANNEL] = 3;
	
	inputs[ARMED_TIM_CHANNEL] = AnalogGet(CFGA2_PIN);
	if (inputs[ARMED_TIM_CHANNEL] < 5) inputs[ARMED_TIM_CHANNEL] = 5;
	
	return result;
}

//---------------------------------------------------
char InputGet(char channel){
	if (inputs[channel] > INPUT_THRESHOLD) return 1;
	return 0;
}

//--------------------------------------------------
void OutProc(void){
	if ((system.state & 1) > 0) {
		system.state = 1;
		OUTPUT_SET;
	} else {
		system.state = 0;
		OUTPUT_OFF;
	}
}

//--------------------------------------------------
void LedProc(void){
	char period;
	char flash;
					
	if (system.mode == MODE_IDLE){	
		flash = LED_SHORT;
		period = LED_PERIOD;	
	}
	if (system.mode == MODE_TO_ARM){
		flash = LED_SHORT;
		period = (LED_SHORT * 3);		
	}
	if (system.mode == MODE_ARMED){
		flash = LED_SHORT;
		period = (LED_PERIOD / 3);
		if (system.alarm) period = (LED_SHORT * 3);
	}
	if (system.mode == MODE_LEARN){		
		flash = LED_LONG;
		period = (LED_LONG * 2);
	}

	if (system.led == 0)  LED_SET;
	if (system.led >= flash) LED_OFF;
	if (system.led < period) {
		system.led++;
	} else {
		system.led = 0;
	}
}

//---------------------------------------------------
void BuzzProc(char time){
	if (time > 0) system.buzz = time;
	
	if (system.buzz > 0){
		system.buzz--;
		BUZZ_SET;
	} else {
		BUZZ_OFF;
	}
}

//---------------------------------------------------
char ConfigGet(char mask){
	if ((inputs[CFG_CHANNEL] & mask) > 0) return 0;
	return 1;
}

//---------------------------------------------------
char UserGet(char act){
	char result = 0;

	if (act > 0) {
		if (act == SW_CHANNEL) {
			result = 1;
			system.ir_hold = IR_HOLD;
		}
	} else {
		if (ScannerCheck(0) == SCAN_COMPLETE) {
			if (ScannerComp() > 0) result = 1;
			system.ir_hold = IR_HOLD;
		}
	}
	return result;
}

//--------------------------------------------------
void HardInit(void){
	ACSR = (1<<ACD);
	PRR = (1<<PRTIM1 | 1<<PRUSI);

	PORTB = (1<<BUZZ_PIN);
	DDRB = (1<<BUZZ_PIN | 1<<OUTPUT_PIN);
	
	DDRA = (1<<LED_PIN);
	PORTA = (1<<DETECT_IN | 1<<REM_PIN | 1<<CFG1_PIN | 1<<CFG2_PIN | 1<<CFG3_PIN);
	
	DIDR0 = (1<<CFGA1_PIN | 1<<CFGA2_PIN);	
}

//---------------------------------------------------
void AnalogInit(void){
	ADCSRA = (1<<ADEN | 1<<ADPS2 | 1<<ADPS1 | 1<<ADPS0);
	ADCSRB = (1<<ADLAR);
	ADMUX = (0<<REFS1 | 0<<REFS0 | CFGA1_PIN);
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC));
}

//---------------------------------------------------
char AnalogGet(char channel){
	while (ADCSRA & (1<<ADSC));
	ADMUX = (0<<REFS1 | 0<<REFS0 | channel);
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC));
	return ADCH;
}